package com.shixing.sxedit;

import com.shixing.sxedit.util.TextUtils;

import java.util.ArrayList;
import java.util.List;

import static com.shixing.sxedit.SXTrack.createNewTrack;

/**
 * 轨道组
 * */

public class SXTrackGroup extends SXBaseImpl {

    /**
     * 获取组 id
     *
     * @return String
     * */
    public String getGroupId() {
        return mGroupId;
    }

    /**
     * start time  display time      duration   disappear time  end time
     *    |             |               |             |            |
     *    |             |-----------------------------|            |
     *    |-------------|-----------------------------|------------|
     * ----------------------------------------------------------------------
     *                 time
     * 在组的某个时间添加一个轨道，如已加入此组或添加到非创建此轨道的合成中返回false
     *
     * <p>
     *     1. 轨道只能添加到创建此轨道的合成中，如果轨道已在另一个组中会自动将轨道从另一个组中移除
     *     2. 轨道在组中的顺序决定了轨道的渲染顺序
     * </p>
     *
     * @param track, {@link SXTrack} 轨道
     * @param displayTime, 轨道的开始显示时间，等价于调用轨道方法 moveToTime
     * @return 添加成功返回true，失败返回false
     */
    public boolean addTrack(SXTrack track, double displayTime) {
        if (mNativeManager > 0 && track != null && track.isValid()) {
            return nAddTrack(mNativeManager, mOwnType.value,  mGroupId, track.getTrackId(), displayTime);
        }
        return false;
    }

    /**
     * 从组中移除轨道
     *
     * @param trackId, 轨道ID
     * @return boolean
     * */
    public boolean removeTrack(String trackId) {
        if (mNativeManager > 0 && !TextUtils.isEmpty(trackId)) {
            return nRemoveTrack(mNativeManager, mOwnType.value, mGroupId, trackId);
        }
        return false;
    }

    /**
     * 清空轨道
     *
     * */
    public void clearTracks() {
        if (mNativeManager > 0) {
            nClearTracks(mNativeManager, mOwnType.value, mGroupId);
        }
    }

    /**
     * 分拆轨道
     *
     * @param trackId, 被分拆的轨道id
     * @param time, 分拆的时间点
     * @return SXTrack, 返回的新轨道
     * */
    /**
     * display time   track     split time     new track    out point
     *   |-----------------------|--------------------------|
     * ----------------------------------------------------------------------
     * 拆分轨道，拆分后的轨道附加效果时间不变，请根据需要做相应的调整
     * <p>
     *     拆分后的新轨道会自动插入到被拆分轨道的下一个位置
     * </p>
     *
     * @param trackId 轨道唯一id
     * @param splitTime 相对于轨道自身display time的局部时间
     * @return {@link SXTrack}, 查找轨道失败或拆分时间不在轨道显示范围内返回 null
     */
    public SXTrack splitTrack(String trackId, double splitTime) {
        if (mNativeManager > 0 && !TextUtils.isEmpty(trackId)) {
            long track = nSplitTrack(mNativeManager, mOwnType.value, mGroupId,  trackId, splitTime);
            if (track > 0) {
                SXTrack.SXTrackOwnType trackOwnType = (mOwnType == SXGroupOwnType.Editor)
                        ? SXTrack.SXTrackOwnType.Editor : SXTrack.SXTrackOwnType.Composite;
                return createNewTrack(track, mNativeManager, trackOwnType);
            }
        }
        return null;
    }

    /**
     * 获取前一个轨道
     *
     * @param trackId, 轨道ID
     * @return SXTrack
     * */
    public SXTrack findPreviousTrack(String trackId) {
        if (mNativeManager > 0 && !TextUtils.isEmpty(trackId)) {
            long track = nFindPreviousTrack(mNativeManager, mOwnType.value, mGroupId,  trackId);
            if (track > 0) {
                SXTrack.SXTrackOwnType trackOwnType = (mOwnType == SXGroupOwnType.Editor)
                        ? SXTrack.SXTrackOwnType.Editor : SXTrack.SXTrackOwnType.Composite;
                return createNewTrack(track, mNativeManager, trackOwnType);
            }
        }
        return null;
    }

    /**
     * 获取指点轨道的下一个轨道
     *
     * @param trackId, 轨道ID
     * @return SXTrack
     * */
    public SXTrack findNextTrack(String trackId) {
        if (mNativeManager > 0 && !TextUtils.isEmpty(trackId)) {
            long track = nFindNextTrack(mNativeManager, mOwnType.value, mGroupId,  trackId);
            if (track > 0) {
                SXTrack.SXTrackOwnType trackOwnType = (mOwnType == SXGroupOwnType.Editor)
                        ? SXTrack.SXTrackOwnType.Editor : SXTrack.SXTrackOwnType.Composite;
                return createNewTrack(track, mNativeManager, trackOwnType);
            }
        }
        return null;
    }

    /**
     * 根据轨道id 获取轨道对象
     *
     * @param trackId, 轨道ID
     * @return SXTrack
     * */
    public SXTrack getTrack(String trackId) {
        if (mNativeManager > 0 && !TextUtils.isEmpty(trackId)) {
            long track = nTrack(mNativeManager, mOwnType.value, mGroupId, trackId);
            if (track > 0) {
                SXTrack.SXTrackOwnType trackOwnType = (mOwnType == SXGroupOwnType.Editor)
                        ? SXTrack.SXTrackOwnType.Editor : SXTrack.SXTrackOwnType.Composite;
                return createNewTrack(track, mNativeManager, trackOwnType);
            }
        }
        return null;
    }


    /**
     * 获取轨道列表
     *
     * @return List {@link SXTrack}
     * */
    public List<SXTrack> getTracks() {
        List<SXTrack> SXTracks = new ArrayList<>();
        SXTrack.SXTrackOwnType trackOwnType = (mOwnType == SXGroupOwnType.Editor)
                ? SXTrack.SXTrackOwnType.Editor : SXTrack.SXTrackOwnType.Composite;
        if (mNativeManager > 0) {
            long[] trackIDs = nTracks(mNativeManager, mOwnType.value, mGroupId);
            if (trackIDs != null) {
                for (long trackID : trackIDs) {
                    SXTrack SXTrack = createNewTrack(trackID, mNativeManager, trackOwnType);
                    SXTracks.add(SXTrack);
                }
            }
        }
        return SXTracks;
    }

    /**
     * 获取时长， 单位：秒
     *
     * @param excludeAudio, 是否不计入音频的时长
     * @return double, 时长
     * */
    public double getDuration(boolean excludeAudio) {
        if (mNativeManager > 0) {
            return nDuration(mNativeManager, mOwnType.value, mGroupId, excludeAudio);
        }
        return 0;
    }

    /**
     * 对组内的 track 排序,并设置 track 前后时间相接, 并应用转场效果
     *
     * @param processTransition, 是否应用转场效果
     * */
    public void arrangeTracks(boolean processTransition) {
        if (mNativeManager > 0) {
            nArrangeTracks(mNativeManager, mOwnType.value, mGroupId, processTransition);
        }
    }


    @Override
    long getNativeHandle() {
        return nGetNativeGroup(mNativeManager, mOwnType.value, mGroupId);
    }


    protected enum SXGroupOwnType {
        UnKnown(0),
        Editor(1),
        Composite(2);

        public final int value;

        SXGroupOwnType(int _value) {
            value = _value;
        }

    }

    long mNativeManager;
    private String mGroupId;
    private SXGroupOwnType mOwnType;

    SXTrackGroup(long group, long nativeManager, SXGroupOwnType type) {
        mNativeManager = nativeManager;
        mGroupId = nGroupID(group);
        mOwnType = type;
    }

    static native String nGroupID(long nativeGroup);
    static native boolean nAddTrack(long nativeManager, int type, String groupID,  String trackID, double startTime);
    static native boolean nRemoveTrack(long nativeManager, int type, String groupID,   String trackId);
    static native void nClearTracks(long nativeManager, int type, String groupID);
    static native long nFindNextTrack(long nativeManager, int type, String groupID,  String trackId);
    static native long nFindPreviousTrack(long nativeManager,  int type, String groupID, String trackId);
    static native long nSplitTrack(long nativeManager, int type, String groupID,  String trackId, double time);
    static native long nTrack(long nativeManager, int type, String groupID,  String trackId);
    static native long[] nTracks(long nativeManager, int type, String groupID);
    static native double nDuration(long nativeManager, int type, String groupID, boolean excludeAudio);
    static native void nArrangeTracks(long nativeManager, int type, String groupID, boolean processTransition);
    static native long nGetNativeGroup(long nativeManager, int type, String groupID);
}
